@@ -44,6 +44,7 @@ import time
 import ldap
 from ldap.ldapobject import ReconnectLDAPObject
 import six
+from six import PY2
 
 log = logging.getLogger(__name__)
 _utf8_encoder = codecs.getencoder('utf-8')
@@ -167,7 +168,8 @@ class ConnectionManager(object):
 
     def _match(self, bind, passwd):
         if passwd is not None:
-            passwd = utf8_encode(passwd)
+            if PY2:
+                passwd = utf8_encode(passwd)
         with self._pool_lock:
             inactives = []
 
@@ -221,7 +223,6 @@ class ConnectionManager(object):
                 raise BackendError('Could not activate TLS on established '
                                    'connection with %s' % self.uri,
                                    backend=conn)
-            conn.start_tls_s()
 
         if bind is not None:
             conn.simple_bind_s(bind, passwd)
@@ -241,7 +242,8 @@ class ConnectionManager(object):
         tries = 0
         connected = False
         if passwd is not None:
-            passwd = utf8_encode(passwd)
+            if PY2:
+                passwd = utf8_encode(passwd)
         exc = None
         conn = None
 
@@ -253,6 +255,11 @@ class ConnectionManager(object):
                 conn.timeout = self.timeout
                 self._bind(conn, bind, passwd)
                 connected = True
+            except ldap.INVALID_CREDENTIALS as error:
+                exc = error
+                log.error('Invalid credentials. Cancelling retry',
+                          exc_info=True)
+                break
             except ldap.LDAPError as error:
                 exc = error
                 time.sleep(self.retry_delay)
@@ -267,7 +274,8 @@ class ConnectionManager(object):
 
         if not connected:
             if isinstance(exc, (ldap.NO_SUCH_OBJECT,
-                                ldap.INVALID_CREDENTIALS)):
+                                ldap.INVALID_CREDENTIALS,
+                                ldap.SERVER_DOWN)):
                 raise exc
 
             # that's something else
