@@ -55,14 +55,29 @@ def _bind_fails2(self, who='', cred='', **kw):
     raise ldap.SERVER_DOWN('LDAP connection invalid')
 
 
+def _bind_fails_invalid_credentials(self, who='', cred='', **kw):
+    raise ldap.INVALID_CREDENTIALS('LDAP connection invalid')
+
+
+def _start_tls_s(self):
+    if self.start_tls_already_called_flag:
+        raise ldap.LOCAL_ERROR
+    else:
+        self.start_tls_already_called_flag = True
+
+
 class TestLDAPConnection(unittest.TestCase):
 
     def setUp(self):
         self.old = ldappool.StateConnector.simple_bind_s
         ldappool.StateConnector.simple_bind_s = _bind
+        self.old_start_tls_s = ldappool.StateConnector.start_tls_s
+        ldappool.StateConnector.start_tls_s = _start_tls_s
+        ldappool.StateConnector.start_tls_already_called_flag = False
 
     def tearDown(self):
         ldappool.StateConnector.simple_bind_s = self.old
+        ldappool.StateConnector.start_tls_s = self.old_start_tls_s
 
     def test_connection(self):
         uri = ''
@@ -115,6 +130,15 @@ class TestLDAPConnection(unittest.TestCase):
         self.assertFalse(cm._pool[0].active)
         self.assertFalse(cm._pool[1].active)
 
+    def test_tls_connection(self):
+        uri = ''
+        dn = 'uid=adminuser,ou=logins,dc=mozilla'
+        passwd = 'adminuser'
+        cm = ldappool.ConnectionManager(uri, dn, passwd, use_pool=True,
+                                        size=2, use_tls=True)
+        with cm.connection():
+            pass
+
     def test_simple_bind_fails(self):
         unbinds = []
 
@@ -133,7 +157,30 @@ class TestLDAPConnection(unittest.TestCase):
         try:
             with cm.connection('dn', 'pass'):
                 pass
-        except ldappool.BackendError:
+        except ldap.SERVER_DOWN:
+            pass
+        else:
+            raise AssertionError()
+
+    def test_simple_bind_fails_invalid_credentials(self):
+        unbinds = []
+
+        def _unbind(self):
+            unbinds.append(1)
+
+        # the binding fails with an LDAPError
+        ldappool.StateConnector.simple_bind_s = _bind_fails_invalid_credentials
+        ldappool.StateConnector.unbind_s = _unbind
+        uri = ''
+        dn = 'uid=adminuser,ou=logins,dc=mozilla'
+        passwd = 'adminuser'
+        cm = ldappool.ConnectionManager(uri, dn, passwd, use_pool=True, size=2)
+        self.assertEqual(len(cm), 0)
+
+        try:
+            with cm.connection('dn', 'pass'):
+                pass
+        except ldap.INVALID_CREDENTIALS:
             pass
         else:
             raise AssertionError()
